<?php

/**
 * BaseHelper.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */
class BaseHelper
{

    /**
     * this var should contain an example href and embedded img tags like:
     * <a href="http://imgchili.net/show/98728/98728125_femjoy_11375010_001.jpg" target="_blank"><img src="http://t11.imgchili.net/98728/98728125_femjoy_11375010_001.jpg" border="0" alt=""/></a>
     * @var snippet
     */
    protected $snippet;

    /**
     * @var verbose either true or false, turns on or off verbose messaging
     */
    protected $verbose;

    /**
     * @var referrer
     */
    protected $referrer;

    /**
     * @var downloader instance of a RipperDownload or similar which implements RipperDownloadInterface
     */
    protected $downloader;

    public function __construct()
    {
    }

    public function setVerbose($verbose) {
        $this->verbose = $verbose;
    }

    public function setDownloader(RipperDownloadInterface $downloader) {
        $this->downloader = $downloader;
    }

    public function setSnippet($snippet) {
        $this->snippet = $snippet;
    }

    public function getSnippet() {
        return $this->snippet;
    }

    public function getImgSrc() {
        $array = parse_array($this->getSnippet(), '<img', '/>');
        $src = $this->getAttribute($array[0], 'src');
        return $src;
    }

    public function getAttribute($html, $attr) {
        $html = str_replace("'", '"', $html);
        $attr .= '="';
        $start = strpos($html, $attr);
        $offset = strlen($attr) + $start;
        $close_quote = strpos($html, '"', $offset);
        $len = $close_quote - $start - strlen($attr);
        $content = substr($html, $offset, $len);
        return $content;
    }

    public function getPage($uri, $opts = null) {
//        out($this->downloader);
        $page = $this->downloader->getPage($uri, $opts);
        return $page;
    }

    public function getPageHtml($uri, $opts = null) {
        $data = $this->downloader->getPage($uri, $opts);
        return $data;
    }
    
    /**
     * sendImages
     * If TRUE, instead of sending the contents of the HREF to the helper, we send the contents of the IMG SRC
     * Currently Picpie and PiczSite process based on the IMG SRC
    */
    public function sendImages() {
        return false;
    }

    /**
     * sendFullSnipet
     * If TRUE, instead of sending the contents of the HREF to the helper, we send the whole HTML snipet, <a... ><img /> </a>
     * Currently we need info from both for Imagetwist.com
    */
    public function sendFullSnipet() {
        return false;
    }
    
    /**
     * getAttributeById
     * $html should be an HTML fragment that contains only one IMG tag
     */
    public function getAttributeById($html, $id, $attribute) {

        if (is_null($html) || is_null($id) || is_null($attribute)) {
            return false;
        }
        $doc = new DOMDocument();
        $doc->loadHTML($html);

        $tag = $doc->getElementById($id);
        $src = $tag->getAttribute($attribute);
        return $src;
    }
}