<?php

/**
 * ImgurComHelper.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

/**
 * Class ImgurComHelper
 *
 * 
 * Test gallery: https://imgur.com/a/ogbRg8j
 * 
 *
 * 
 */
class ImgurComHelper extends BaseHelper
{

    public function __construct($client_id) {
        $this->client_id = $client_id;
    }

    /**
     * createImageArray
     * we start with something like: 
     * {"hash":"IjSgwK1","title":"","description":null,"has_sound":false,"width":1062,"height":1600,"size":185570,"ext":".jpg","animated":false,"prefer_video":false,"looping":false,"datetime":"2019-02-01 00:13:39"}
     * and need to return something like: 
     * https://i.imgur.com/IjSgwK1.jpg
     * @todo this method needs to be updated to use the Imgur API.  The album_images information isn't available when doing file_get_contents
     * @param $html the full HTML of the Imgur album page (like view source)
     * @return array Return array of URIs that point to the images
     */
//    public function createImageArray($html) {
//
//        $json = json_decode($html);
//        $images = $json->album_images->images;
//
//        foreach($images as $image_object) {
//            //$array = $this->convertToArray($piece);
//            $image['link'] = 'https://i.imgur.com/'. $image_object->hash . $image_object->ext;
//            if (empty($image_object->title)) {
//                $image['name'] = $image_object->hash;
//            } else {
//                $image['name'] = $image_object->title;
//            }
//            $retval[] = $image;
//        }
//        return $retval;
//    }
//
//    public function convertToArray($piece) {
//        out($piece);
//        $piece = str_replace('"', '', $piece);
//        $parts = explode(',', $piece);
//        foreach ($parts as $part) {
//            $bits = explode(":", $part);
//            $retval[$bits[0]] = $bits[1];
//        }
//        return $retval;
//    }

    /**
     * process
     * @param $item an array of information about image on imgur to process
     * @return array Return array with 'image' and 'name' as the keys
     */
    public function process($item) {
        if (empty($item['title'])) {
            $title = $item['id'] .'.jpg';
        } else {
            $title = $item['title'];
        }
        $image['name'] = $title;
        $image['image'] = $this->getPage($item['link']);
        return $image;
    }

    public function getImagesForUri($uri) {
        $hash = $this->extractHashFromUri($uri);
        $album = $this->imgurApiAlbumGet($hash);
        $data = json_decode($album);
        $images = $data->data->images;
        foreach ($images as $image) {
            $image_info[] = [
                'id' => $image->id,
                'title' => $image->title,
                'link' => $image->link
            ];
        }
        return $image_info;
    }

    /**
     * @param $uri the string uri of the resource we are trying to download, like: https://imgur.com/a/ogbRg8j
     * @return string should return the hash value found in the uri, for example 'ogbRg8j'
     * @todo will probably need to update to handle different formats of uri endpoints
     */
    public function extractHashFromUri($uri) {
        $parts = parse_url($uri);
        $hash = str_replace('/a/', '', $parts['path']);
        return $hash;
    }

    public function imgurApiAlbumGet($album_hash) {
        $url = "https://api.imgur.com/3/album/". $album_hash;
        $retval = $this->imgurCurl($url);
        return $retval;
    }

    public function imgurApiAlbumGetImages($album_hash) {
        $url = "https://api.imgur.com/3/album/". $album_hash."/images";
        $retval = $this->imgurCurl($url);
        return $retval;
    }

    public function imgurApiImageGet($image_hash) {
        $url = "https://api.imgur.com/3/image/". $image_hash;
        $retval = $this->imgurCurl($url);
        return $retval;
    }

    public function imgurCurl($url) {
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => array(
                "Authorization: Client-ID ". $this->client_id
            ),
        ));

        $response = curl_exec($curl);

        curl_close($curl);
        return $response;
    }
}