<?php
/**
 * RipperDownload.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

class RipperDownload implements RipperDownloadInterface
{
    protected $verbose;
    protected $user_agent;
    protected $referrer = null;
    protected $username = null;
    protected $password = null;

    public function __construct($verbose)
    {
        $this->verbose = (bool)$verbose;
    }

    public function init($values) {
        foreach ($values as $key=>$value) {
            $this->$key = $value;
        }
    }

    public function getPage($uri, $opts = null)
    {
        verbose('getPage - looking for: '. $uri, $this->verbose);
        if(empty($this->user_agent)) {
            $this->user_agent = setHttpUserAgent();
        }
        verbose('using user_agent: '. $this->user_agent, $this->verbose);
        $arrContextOptions=array(
            "ssl"=>array(
                "allow_self_signed" => true,
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
            "http" => array(
                "user_agent" => $this->user_agent
            )
        );
        if (empty($uri)) {
            return false;
        }
        $page = @file_get_contents($uri, false, stream_context_create($arrContextOptions));
        if(empty($http_response_header)) {
            $http_response_header = array('HTTP/1.1 400 Bad request');
        }
        $headers = $this->parseHeaders($http_response_header);
        if ($this->isHttpError($headers['response_code'])) {
            $this->errors[] = ['headers'=>$headers, 'uri'=>$uri];
            $page = false;
        }
        return $page;
    }

    public function getPageHtml($uri, $opts = null)
    {
        verbose('getPageHtml - looking for: '. $uri, $this->verbose);
        $ch = curl_init();
        $timeout = 5;
        if (!is_null($this->referrer)) {
            curl_setopt($ch, CURLOPT_REFERER, $this->referrer);
        }
        curl_setopt($ch, CURLOPT_URL, $uri);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
        curl_setopt($ch, CURLOPT_COOKIESESSION, true);
        curl_setopt($ch, CURLOPT_COOKIEJAR, 'cookie-vipergirls.txt');
        curl_setopt($ch, CURLOPT_COOKIEFILE, 'cookie-vipergirls.txt');

        $data = curl_exec($ch);
        curl_close($ch);
        return $data;
    }

    public function postForm($uri, $opts = null)
    {
//        out($opts);
        verbose('postForm: '. $uri, $this->verbose);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_URL,$uri);
        curl_setopt($ch, CURLOPT_COOKIESESSION, true);
        curl_setopt($ch, CURLOPT_COOKIEJAR, 'cookie-vipergirls.txt');  //could be empty, but cause problems on some hosts
        curl_setopt($ch, CURLOPT_COOKIEFILE, 'cookie-vipergirls.txt');  //could be empty, but cause problems on some hosts

        foreach ($opts as $key=>$value) {
            switch ($key) {
                case 'CURLOPT_POSTFIELDS':
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $value);
                    break;

                case 'CURLOPT_REFERER':
                    curl_setopt($ch, CURLOPT_REFERER, $value);
                    break;

                case 'CURLOPT_HEADER':
                    curl_setopt($ch, CURLOPT_HEADER, $value);
                    break;

                default:
                    curl_setopt($ch, intval($key), $value);
                    break;
            }
        }
//out(curl_getinfo($ch));

        $server_output = curl_exec ($ch);

        curl_close ($ch);
        return $server_output;
    }

    public function loginToSite() {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://www.vipergirls.to/login.php?do=login');
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_USERAGENT,$this->user_agent);
        curl_setopt($ch, CURLOPT_POST, true);
//        curl_setopt($ch, CURLOPT_POSTFIELDS, "vb_login_username=Laxaeman&vb_login_password=xBJuCX88HAqkP69oebq0rjyCw4LwAwZp");
        // url=%2Fthreads%2F4623994-PJGirls-2019&
        curl_setopt($ch, CURLOPT_POSTFIELDS, "do=login&vb_login_md5password=&vb_login_md5password_utf=&s=&securitytoken=guest&vb_login_username=".$this->username."&vb_login_password=".$this->password);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_COOKIESESSION, true);
        curl_setopt($ch, CURLOPT_COOKIEJAR, 'cookie-vipergirls.txt');  //could be empty, but cause problems on some hosts
        curl_setopt($ch, CURLOPT_COOKIEFILE, 'cookie-vipergirls.txt');  //could be empty, but cause problems on some hosts
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
//out(curl_getinfo($ch));

        $retval = curl_exec($ch);

        if (curl_error($ch)) {
            echo curl_error($ch);
        }
        curl_close($ch);
//        out($retval);
        return $retval;
    }

    public function parseHeaders( $headers )
    {
        verbose('parseHeaders: '. implode(', ', $headers), $this->verbose);
        $head = array();
        foreach( $headers as $k=>$v )
        {
            $t = explode( ':', $v, 2 );
            if( isset( $t[1] ) )
                $head[ trim($t[0]) ] = trim( $t[1] );
            else
            {
                $head[] = $v;
                if( preg_match( "#HTTP/[0-9\.]+\s+([0-9]+)#",$v, $out ) )
                    $head['response_code'] = intval($out[1]);
            }
        }
        return $head;
    }

    /**
     * if response code is 4xx or 5xx it's an error of some kind
     * probably should update this to be a bit more robust
     * @param $code
     * @return boolean
     */
    public function isHttpError($code) {
        if($code > 399) {
            return true;
        }
        return false;
    }
}