<?php
/**
 * RipperHelp.php
 *
 * @version $Id: $
 * @package ripper
 * @copyright 2019
 */

class RipperHelp
{

    public function __construct()
    {
        require_once 'app/rip_config.php';
        $this->current_version = $this->getCurrentReleaseValue();
    }

    public function getCurrentReleaseValue() {
        require_once 'releases.php';
        $current = array_pop($releases);
        return $current['version'];
    }
    public function displayCliHelp()
    {
        $content = $this->buildForCli($this->helpContents());
        $hosts = $this->getSupportedHosts();
        echo PHP_EOL;
        echo 'Supported file hosts: '. PHP_EOL;
        foreach ($hosts as $host) {
            echo $host . PHP_EOL;
        }
    }

//    public function displayHtmlHelp() {
//        $content = $this->buildReadme($this->helpContents());
//        var_dump($content);
//    }

    public function writeReadme($type = 'html') {
        if($type == 'html') {
            $file = 'readme.html';
            $content = $this->buildReadme($this->helpContents());
        } else {
            $file = 'readme.md';
            $content = $this->buildReadmeMarkdown($this->helpContents());
        }
        $rs = file_put_contents($file, $content);
        var_dump($rs);
    }

    public function buildForCli($content)
    {
        foreach ($content as $line) {
            if (isset($line['dest']) && ($line['dest'] == 'r')) {
                continue;
            }
            if ($line['content'] == 'newline') {
                echo PHP_EOL;
            } else {
                echo $line['content'] . PHP_EOL;
            }
        }
    }

    public function getSupportedHosts($asHtml = false)
    {
        $domains = [];
        $d = dir("app/Helpers");
        while (false !== ($entry = $d->read())) {
            $ignore = ['.', '..'];
            if (in_array($entry, $ignore)) {
                continue;
            }
            $domain = str_replace('Helper.php', '', $entry);
            $search = ['Cc', 'Com', 'Gy', 'Me', 'Net', 'Org', 'Se', 'Site', 'To', 'Yt'];
            $replace = ['.cc', '.com', '.gy', '.me', '.net', '.org', '.se', '.site', '.to', '.yt'];
            $domains[] = strtolower(str_replace($search, $replace, $domain));
            sort($domains);
        }
        $d->close();
        if ($asHtml) {
            foreach ($domains as $domain) {
                $retval[] = '<li>' . $domain . '</li>';
            }
            return $retval;
        }
        return $domains;
    }

    public function buildReadme($content)
    {
        $html = '<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>LaxaeRipper</title>
</head>
';
        foreach ($content as $line) {
            if ($line['content'] == 'newline') {
                $html .= '<p><br /></p>'.PHP_EOL;
            } else {
                switch ($line['level']) {
                    case 'h1':
                        $prepend = '<h1>';
                        $append = '</h1>';
                        break;
                    case 'h2':
                        $prepend = '<h2>';
                        $append = '</h2>';
                        break;
                    case 'h3':
                        $prepend = '<h3>';
                        $append = '</h3>';
                        break;
                    case 'h4':
                        $prepend = '<h4>';
                        $append = '</h4>';
                        break;
                    case 'code':
                        $prepend = '<pre lang="text"><code>';
                        $append = '</code></pre>';
                        break;
                    case 'body_indent':
                        $prepend = '<p style="margin-left: 40px">';
                        $append = '</p>';
                        break;
                    default:
                        $prepend ='<p>';
                        $append = '</p>';
                        break;
                }
                if(isset($line['escape']) && !$line['escape']) {
                    $html .= $prepend . $line['content'] . $append .PHP_EOL;
                } else {
                    $html .= $prepend . htmlspecialchars($line['content']) . $append .PHP_EOL;
                }
            }
        }
        $hosts = $this->getSupportedHosts(true);
        $html .= '<h2>Supported Hosts</h2>';
        $html .= '<p>The following is a list of the currently supported hosts</p>';
        $html .= '<ul>';
        foreach ($hosts as $host) {
            $html .= $host;
        }
        $html .= '</ul>';
        $html .= '</body>
</html>';
        return $html;
    }

    public function buildReadmeMarkdown($content) {
        $mark = '';

        foreach ($content as $line) {
            if ($line['content'] == 'newline') {
                $mark .= '  ' . PHP_EOL;
            } else {
                switch ($line['level']) {
                    case 'h1':
                        $prepend = '# ';
                        $append = '';
                        break;
                    case 'h2':
                        $prepend = '## ';
                        $append = '';
                        break;
                    case 'h3':
                        $prepend = '### ';
                        $append = '';
                        break;
                    case 'h4':
                        $prepend = '#### ';
                        $append = '';
                        break;
                    case 'code':
                        $prepend = '`';
                        $append = '`  ';
                        break;
                    case 'body_indent':
                        $prepend = '    ';
                        $append = '';
                        break;
                    default:
                        $prepend = '';
                        $append = '  '.PHP_EOL;
                        break;
                }
//                if (isset($line['escape']) && !$line['escape']) {
                    $mark .= $prepend . $line['content'] . $append . PHP_EOL;
//                } else {
//                    $mark .= $prepend . htmlspecialchars($line['content']) . $append . PHP_EOL;
//                }
            }
        }
        return $mark;
    }

    public function helpContents()
    {
        $config = confItems();
        $contents[] = ['level' => 'h1', 'content' => 'Help information for LaxaeRipper'];
        $contents[] = ['level' => 'body', 'content' => 'This application will download (or rip) images from various image hosts.  Mostly it is intended to be used on https://vipergirls.to'];
        $contents[] = ['level' => 'body', 'content' => 'Current version: '. $this->current_version];
        $contents[] = ['level' => 'h2', 'content' => 'Installation', 'dest' => 'r'];
        $contents[] = ['level' => 'h3', 'content' => 'Windows Installation', 'dest' => 'r'];
        $contents[] = ['level' => 'h4', 'content' => 'Prerequisites', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'You must have php installed and running on your computer.', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'To verify that you do have php installed, open a Command prompt and type:', 'dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => 'php -v', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'You should see something like:', 'dest' => 'r'];
        $contents[] = ['level' => 'image', 'content' => '<img src="assets/phpv.png">', 'dest' => 'r', 'escape'=>false];
        $contents[] = ['level' => 'body', 'content' => 'If you don\'t see something similar, then you will need to install php and add it to your PATH', 'dest'=>'r'];
        $contents[] = ['level' => 'body', 'content' => 'To install php: <a href="https://www.php.net/manual/en/install.windows.php">https://www.php.net/manual/en/install.windows.php</a>', 'dest'=>'r', 'escape'=>false];
        $contents[] = ['level' => 'body', 'content' => 'To add php to your PATH: <a href="https://www.sunant.com/running-php-from-windows-command-line/">https://www.sunant.com/running-php-from-windows-command-line/</a>', 'dest'=>'r', 'escape'=>false];
        $contents[] = ['level' => 'h3', 'content' => 'Mac Installation', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'You must have php installed and running on your computer.  If you don\'t you can use these instructions to get it installed', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'To install php: <a href="https://www.php.net/manual/en/install.macosx.php">https://www.php.net/manual/en/install.macosx.php</a>', 'dest' => 'r', 'escape'=>false];

        $contents[] = ['level' => 'h3', 'content' => 'Unzip the laxaeripper-x.y.z.zip file', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'Extract the contents of the zip file, it can be located pretty much anywhere on your computer. You should have a directory and file structure like this:','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => 'laxaeripper/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── app/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '│   └── Helpers/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── assets/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── check.php','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── conf/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '│   └── config.php','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── cookie_vipergirls.txt','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── example_list.csv','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── example_list.php','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── history/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── images/','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── laxaeripper.sqlite','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── license.md','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── readme.html','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '├── readme.md','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => '└── rip.php','dest' => 'r'];
        $contents[] = ['level' => 'h3', 'content' => 'Uninstall', 'dest' => 'r'];
        $contents[] = ['level' => 'body', 'content'=>'To uninstall simply delete the laxaeripper directory and the zip file.  LaxaeRipper doesn\'t install anything or put anything else on your computer.','dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'newline','dest' => 'r'];
        $contents[] = ['level' => 'h2', 'content' => 'Usage'];
        $contents[] = ['level' => 'body', 'content' => 'To run LaxaeRipper, open a terminal window or Command Prompt and navigate to the laxaeripper folder.  Then type:', 'dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => 'php rip.php', 'dest' => 'r', 'escape'=>false];
        $contents[] = ['level' => 'body', 'content' => 'You should see something like this:', 'dest' => 'r'];
        $contents[] = ['level' => 'image', 'content' => '<img width="400" src="assets/menu-1.0.0.png">', 'dest' => 'r', 'escape'=>false];

        $contents[] = ['level' => 'body', 'content' => 'Select one of the variety of ways to download images or control the application by clicking the corresponding key on your keyboard.'];

        $contents[] = ['level' => 'body', 'content' => '[1] Rip images in thread - one or most posts'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to download all the images in one or more posts in a thread.  Most of the time, I use this option.  For example, this page: https://vipergirls.to/threads/5262129-Taylor-Nature-Queen-(May-06-2020)-82X currently contains three different posts of the same photo set.  LaxaeRipper will try to figure out the title for each post and will do some clean up of the title.  This option is probably the one you will use most often.  The easiest way to use this option is to copy the URL from your browser and paste it when prompted.'];

        $contents[] = ['level' => 'body', 'content' => '[2] Rip images in post'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to download all the images in a single post of a thread.  This is helpful if there are a large number of posts in a thread (for example the "collections" thread can have a huge number of posts). To use this option, right click on the post number in the post header, it will be something like #121,(see image below) and copy the URI to the post.  In Chrome look for Copy Link Address.  In Firefox look for Copy Link Location.  Then paste it when prompted'];
        $contents[] = ['level' => 'image', 'content' => '<img width="350" src="assets/post_number.png">', 'dest' => 'r', 'escape'=>false];

        $contents[] = ['level' => 'body', 'content' => '[3] Rip images from threads listed in a file'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to download all threads or posts which are stored in a file.  The file must be in the root of the application and it must follow the format of either of the example files: example_list.php or example_list.csv.  Albums in the file are treated by LaxaeRipper the same was as Option 2.  You can use either of the example_list files as the basis of your file.  The files must have a title and URI and optionally a \'start\' value.  If a start value is set, we will start the download at that point in the list.  If there are 100 images to download and the start number is 25, then we will skip the first 24 images and start downloading at number 25.'];

        $contents[] = ['level' => 'body', 'content' => '[4] Rip images from a pasted list'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to download an arbitrary list of images.  Best way to get the list is to view the source of the page and copy the links for the images that you want.  The image links will be between <blockquote class="postcontent restore "> and </blockquote>'];

        $contents[] = ['level' => 'body', 'content' => '[6] History'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to view your download history.  You can also create an HTML report for your download history.'];
        $contents[] = ['level' => 'body_indent', 'content' => '    When you are in the History section you can:'];
        $contents[] = ['level' => 'body_indent', 'content' => '    [1] view history, which will display a table of your downloads', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    [2] generate HTML report, which will create an HTML file named something like: history-2019-11-03.html with a history of your downloads.  This file will be located in the history folder.', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    [x] clear history, this will delete your download activity/history from the SQLite database.', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    History is turned on by default, you can change that in Config by changing log_downloads to FALSE', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    The history of your downloads is stored in a SQLite database called \'laxaeripper\'. You can use any app that interacts with SQLite databases to view the contents.', 'escape'=>false];
        $contents[] = ['level' => 'body', 'content' => '[7] Update'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to update Laxaeripper to a newer version, if one is available.'];

        $contents[] = ['level' => 'body', 'content' => '[8] Config'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to configure Laxaeripper.  You can use this to create the config file, edit it and view your current config.'];
        $contents[] = ['level' => 'body_indent', 'content' => '    When you are in the Config section you can:'];
        $contents[] = ['level' => 'body_indent', 'content' => '    [ 1 ] to edit, which allows you to edit your current config file.', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    [ 2 ] to replace, which allows you to replace your current config file.', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    [ 3 ] view current config values, which allows you to view your current config file settings.', 'escape'=>false];
        $contents[] = ['level' => 'body_indent', 'content' => '    [ 9 ] test login, use this to confirm that LaxaeRipper can log into ViperGirls with your credentials.', 'escape'=>false];

        $contents[] = ['level' => 'body', 'content' => '[9] Help Information'];
        $contents[] = ['level' => 'body_indent', 'content' => '    Use this to view the help information.'];
        $contents[] = ['level' => 'body', 'content' => 'newline'];
        $contents[] = ['level' => 'h2', 'content' => 'FAQ'];
        $contents[] = ['level' => 'body', 'content' => 'Filenames.  LaxaeRipper will use the file names supplied by the image host for the images that are being downloaded.  Sometimes those are kind of nasty, like this one: 174ffa1269420444.jpg  You can configure Laxaeripper to prepend or append to the file name in Config.'];
        $contents[] = ['level' => 'body', 'content' => 'newline'];
        $contents[] = ['level' => 'h2', 'content' => 'Flags'];
        $contents[] = ['level' => 'body', 'content' => '-v sets LaxaeRipper to run in verbose mode, which will override whatever is in config'];
        $contents[] = ['level' => 'body', 'content' => 'newline'];
        $contents[] = ['level' => 'h2', 'content' => 'Config'];
        $contents[] = ['level' => 'body', 'content' => 'You can use Config to change the behavior of LaxaeRipper.  The config values are stored in conf/config.php'];
        $contents[] = ['level' => 'body', 'content' => 'The following is a list of the current config items and their default values'];
//        $contents[] = ['level' => 'body', 'content' => 'newline'];

        foreach ($config as $conf_item) {
            $contents[] = ['level' => 'body', 'content' => $conf_item['key'] .': '. $conf_item['note']];
        }

        $contents[] = ['level' => 'body', 'content' => 'newline'];
        $contents[] = ['level' => 'h2', 'content' => 'Additional thoughts'];
        $contents[] = ['level' => 'body', 'content' => 'Depending on how many images you\'re planning to download at a time you might need to change the memory-limit for php.'];
        $contents[] = ['level' => 'code', 'content' => 'memory_limit = 128M'];
        $contents[] = ['level' => 'body', 'content' => 'Note: I\'ve downloaded over 1000 images at a time with the memory_limit at 128M without problems.'];
//        $contents[] = ['level' => 'body', 'content' => ''];

        $contents[] = ['level' => 'h2', 'content' => 'Help/Support'];
        $contents[] = ['level' => 'body', 'content' => 'If you need help or support with this application, please feel free to ask me <a href="https://vipergirls.to/members/288419-Laxaeman">Laxaeman</a>.  I\'ll do what I can to help.', 'escape'=>false];
        $contents[] = ['level' => 'body', 'content' => 'If you find a bug (and I\'m sure you will eventually) please create a new post on the thread on vipergirls.to for the bug.'];
        $contents[] = ['level' => 'body', 'content' => 'As I add new features and fix bugs, I\'ll create a new post for the updated version, in the thread on vipergirls.to'];

        $contents[] = ['level' => 'h2', 'content' => 'License', 'dest'=>'r'];
        $contents[] = ['level' => 'html', 'content' => '<a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/"><img alt="Creative Commons License" style="border-width:0" src="https://i.creativecommons.org/l/by-nc-sa/4.0/88x31.png" /></a><br />This work is licensed under a <a rel="license" href="http://creativecommons.org/licenses/by-nc-sa/4.0/">Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License</a>', 'dest'=>'r', 'escape'=>false];
        $contents[] = ['level' => 'h3', 'content' => 'PHP Requirements','dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'Make sure that file_get_contents() can open a https address by uncommenting this line in your php.ini file:','dest' => 'r'];
        $contents[] = ['level' => 'code', 'content' => ';extension=php_openssl.dll','dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'Uncomment the line by removing the semi-colon.','dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'newline'];        $contents[] = ['level' => 'h3', 'content' => 'cURL','dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'open a terminal or Command Prompt and run the check.php file that\'s located in the laxaeripper folder.  If you have cURL installed and configured, then you\'re good to go.  Otherwise you will need to install it.','dest' => 'r'];
        $contents[] = ['level' => 'body', 'content' => 'To install cURL on a Windows10 machine I did the following:<br>
    Download from https://curl.haxx.se/download.html<br>

    Unzip the downloaded file and place the files somewhere on your computer.  For me I put them in a directory c:\dev\curl<br>

    Go to System Properties...Advanced...Environment Variables, click on "Path" and then Edit.  Add curl to the path by clicking New then typing in the location of the exe.  For me that was c:\dev\curl\curl.exe then click OK.  Then close out of System Properties.<br>

    Open up a Command Prompt and type \'curl -help\'  (without the quotes) you should see a bunch of information about cURL.<br>

    If you don\'t then cURL isn\'t installed correctly.  Most likely you have the location incorrect in the path.  Which means fixing your PATH.<br>

    Now we need to get the digital certifcates installed so we can read SSL sites.<br>

    Go to: https://curl.haxx.se/docs/caextract.html and download the cacert.pem file.<br>
    Save the cacert.pem into your curl directory.<br>

    Next edit the php.ini file, by opening it in your favorite text editor.<br>
    Look for it in the php folder.  If you don\'t have a php.ini file, then open the php.ini-development and save it as php.ini<br>
    Next search for<br>
<pre>    ;extension=curl</pre>
    and remove the leading semi-colon.

    You might have to replace curl with the absolute path to the curl.exe.  In my case the line looks like this:
<pre>    extension=C:\dev\php\ext\php_curl.dll</pre>

    Now look for:
<pre>    ;curl.cainfo=</pre>
    and remove the leading semi-colon, and add in the absolute path to the cacert.pem file that you just downloaded.
    In my case the line looks like this:
<pre>   curl.cainfo="C:\dev\curl\cacert.pem"</pre>

    Next look for:
<pre>    allow_url_include = Off</pre>
    and change it to:
<pre>    allow_url_include = On</pre>

    Finally look for:
<pre>    ;extension=openssl</pre>
    and change it to include the path to your php ext folder:
<pre>    extension=C:\dev\php\ext\php_openssl.dll</pre>
    Change the path to match where you have php installed.

    Save and exit the file.','dest' => 'r', 'escape'=>false];
        $contents[] = ['level' => 'body', 'content' => 'To ensure you have cURL installed correctly you can run the check.php file.', 'dest' => 'r'];

        $contents[] = ['level' => 'body', 'content' => 'If you need to install sqlite3 on your computer, go to https://www.sqlite.org/downloads and follow the instructions for installing sqlite3.s'];
//        $contents[] = ['level' => '', 'content' => ''];
        return $contents;
    }
}