<?php
/**
 * RipperKittyKats.php
 *
 * @version $Id: $
 * @package ripper
 * @copyright 2019
 */

/**
 * this uri:
 * http://kitty-kats.net/threads/katya-v-pileias-x131.2691262/
 *
 */



class RipperKittyKats implements RipperDownloadInterface
{

    protected $verbose;

    protected $referer = null;

    public function __construct($verbose)
    {
        $this->verbose = (bool)$verbose;
    }

    public function init($values) {
        verbose(__METHOD__.' - setting for: '. var_export($values, true), $this->verbose);

        foreach ($values as $key=>$value) {
            $this->$key = $value;
        }
    }

    /**
     * @param $uri string expecting this to be URI to the thread, like: http://kitty-kats.net/threads/katya-v-pileias-x131.2691262/
     * @param array $opts not implemented, should be null for now
     * @return bool|string
     */
    public function getUri($uri, $opts = null) {
        verbose(__METHOD__.' - looking for: '. $uri, $this->verbose);
        return $this->getFileWithCurl($uri, $opts);
    }

    public function getPage($uri, $opts = null){
        verbose(__METHOD__.' - looking for: '. $uri, $this->verbose);
        return $this->getFile($uri, $opts);
    }

    public function getFileWithCurl($uri, $opts = null) {
        $ch = curl_init();
        $timeout = 5;
        if (!is_null($this->referer)) {
            curl_setopt($ch, CURLOPT_REFERER, $this->referer);
        }
        curl_setopt($ch, CURLOPT_URL, $uri);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
        curl_setopt($ch, CURLOPT_COOKIESESSION, true);
        curl_setopt($ch, CURLOPT_COOKIEJAR, 'cookie-kittykats.txt');
        curl_setopt($ch, CURLOPT_COOKIEFILE, 'cookie-kittykats.txt');

        $data = curl_exec($ch);
        curl_close($ch);
        return $data;
    }

    /**
     * this is the first method called by rip_album
     * @param $uri
     * @param null $opts
     */
    public function getPageHtml($uri, $opts = null) {
        verbose(__METHOD__ .' - looking for: '. $uri, $this->verbose);
        $api_uri = $this->getApiUri($uri);
        $page = $this->getFile($api_uri, $opts);

        return $page;
    }

    public function getFile($uri, $opts = null) {
        if(empty($this->user_agent)) {
            $this->user_agent = setHttpUserAgent();
        }
        verbose('using user_agent: '. $this->user_agent, $this->verbose);
        $arrContextOptions=array(
            "ssl"=>array(
                "allow_self_signed" => true,
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ),
            "http" => array(
                "user_agent" => $this->user_agent
            )
        );
        if (empty($uri)) {
            return false;
        }
        $page = @file_get_contents($uri, false, stream_context_create($arrContextOptions));
        if(empty($http_response_header)) {
            $http_response_header = array('HTTP/1.1 400 Bad request');
        }
        $headers = $this->parseHeaders($http_response_header);
        if ($this->isHttpError($headers['response_code'])) {
            $this->errors[] = ['headers'=>$headers, 'uri'=>$uri];
            $page = false;
        }
        return $page;
    }

    public function postForm($uri, $opts=null) {
        verbose(__METHOD__ .' - looking for: '. $uri, $this->verbose);
        require_once 'RipperDownload.php';

        $rd = new \RipperDownload($this->verbose);
        return $rd->postForm($uri, $opts);
    }

    /**
     * receiving something like:
     * http://kitty-kats.net/threads/katya-v-pileias-x131.2691262/
     * should return:
     * ?
     * @todo may not need this method with KittyKats
     * @param $uri
     */
    public function getApiUri($uri) {
        verbose('getApiUri: '. $uri, $this->verbose);

        $pieces = explode('threads/', $uri);
        $parts = explode('-', $pieces[1]);
        $id = reset($parts);
        //return 'https://vipergirls.to/vr.php?t='. $id;
    }

    public function parseHeaders( $headers )
    {
        verbose('parseHeaders: '. implode(', ', $headers), $this->verbose);
        $head = array();
        foreach( $headers as $k=>$v )
        {
            $t = explode( ':', $v, 2 );
            if( isset( $t[1] ) )
                $head[ trim($t[0]) ] = trim( $t[1] );
            else
            {
                $head[] = $v;
                if( preg_match( "#HTTP/[0-9\.]+\s+([0-9]+)#",$v, $out ) )
                    $head['response_code'] = intval($out[1]);
            }
        }
        return $head;
    }

    /**
     * if response code is 4xx or 5xx it's an error of some kind
     * probably should update this to be a bit more robust
     * @param $code
     * @return boolean
     */
    public function isHttpError($code) {
        if($code > 399) {
            return true;
        }
        return false;
    }

}