<?php
    
/**
 * RipperUpdater.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */
    
class RipperUpdater {
    
    protected $verbose = false;

    /**
     * set to true to do a smoke test instead of an actual update
     * set to false to do the full update
     * @var bool
     */
    protected $smoke = false;
//    protected $smoke = true;

    const NO_FILES = 'no files to process';

    public function __construct($server) {
        $this->server = $server;
    }

    function getCurrentVersion() {
        return '2.0.9';
        verbose(__METHOD__ .' getting current version number', $this->getVerbose());
        $version = file_get_contents($this->server::UPDATE_SERVER . $this->server->getVersionFile('2.0.9'));
        if (!$version) {
            verbose(__METHOD__ .' failed to find version file', $this->getVerbose());
        }
        return $version;
    }

    public function upgrade() {
        verbose(__METHOD__ .' starting ', $this->getVerbose());
        $manifest = $this->getFile($this->server::UPDATE_SERVER . $this->server->getManifestFile('manifest.json.oneofeach'));
        verbose('manifest.json - found these files to update: '. $manifest, $this->getVerbose());
        $files = json_decode($manifest);

        $this->manifest = $files;

        foreach ($files as $section=>$files) {
            switch ($section) {
                case 'new':
                $rs['new'] = $this->newFiles($files);
                break;
                case 'delete':
                $rs['delete'] = $this->deleteFiles($files);
                break;
                case 'update':
                default:
                $rs['update'] = $this->updateFiles($files);
                break;
            }
        }
        return $rs;
    }

    public function verify($manifest, $results) {
        $sections = ['update', 'delete', 'new'];
        $expect = [];
        foreach ($sections as $section) {
            if (!empty($manifest->{$section})) {
                foreach ($manifest->{$section} as $item) {
                    $expect[$section][$item->target] = true;
                }
            } else {
                $expect[$section] = RipperUpdater::NO_FILES;
            }
        }
        if($expect == $results) {
            $retval = true;
        } else {
            $retval = false;
        }
        return $retval;
    }
    
    public function getFile($file) {
        verbose(__METHOD__ .' looking for: '. $file, $this->getVerbose());
        return file_get_contents($file);
    }
    
    public function newFiles($files) {
        verbose(__METHOD__, $this->getVerbose());
        if (empty($files)) {
            return RipperUpdater::NO_FILES;
        }
        foreach ($files as $file) {
            if ($this->smoke) {
                echo 'SMOKE: creating new file: '. $file->target .' from '. $this->server::UPDATE_SERVER . $file->source . PHP_EOL;
                $rs[$file->target] = true;
            } else {
                $contents = $this->getFile($this->server::UPDATE_SERVER . $file->source);
//var_dump($contents); exit;
                verbose('writing: '. $file->target, $this->getVerbose());
                $rs[$file->target] = file_put_contents($file->target, $contents);
            }
        }
        return $rs;
    }
    
    public function deleteFiles($files) {
        verbose(__METHOD__, $this->getVerbose());
        if (empty($files)) {
            return RipperUpdater::NO_FILES;
        }
        foreach ($files as $file) {
            if ($this->smoke) {
                echo 'SMOKE: deleting file: '. $file->target .' in '. __DIR__ . PHP_EOL;
                $rs[$file->target] = true;
            } else {
                verbose('deleting: '. SITE_ROOT . $file->target, $this->getVerbose());

                if (!file_exists(SITE_ROOT . $file->target)) {
                    verbose('could not find: '. SITE_ROOT . $file->target . ' in '. SITE_ROOT, $this->getVerbose());
                    $rs[$file->target] = false;
                } else {
                    verbose('found: '. $file->target, $this->getVerbose());
                    $rs[$file->target] = unlink($file->target);
                }
            }
        }
        return $rs;
    }
    
    public function updateFiles($files) {
        verbose(__METHOD__, $this->getVerbose());
        if (empty($files)) {
            return 'no files to process';
        }
        foreach ($files as $file) {
            if ($this->smoke) {
                echo 'SMOKE: update file: '. $file->target .' with '. $this->server::UPDATE_SERVER . $file->source .PHP_EOL;
                //echo $this->getFile($this->server::UPDATE_SERVER . $file->source) . PHP_EOL;
                $rs[$file->target] = true;
            } else {
                $contents = $this->getFile($this->server::UPDATE_SERVER . $file->source);
                verbose('creating a backup of: '. $file->target, $this->getVerbose());
                copy($file->target, $file->target .'.bak');
                verbose('updating: '. $file->target, $this->verbose);
                $rs[$file->target] = file_put_contents($file->target, $contents);
            }
        }
        return $rs;
    }
    
    public function setVerbose($verbose = false) {
        if ($verbose) {
            $this->verbose = true;
        } else {
            $this->verbose = false;
        }
    }

    public function getVerbose() {
        return $this->verbose;
    }
}
    
