<?php
/**
 * setHttpUserAgent
 */
function setHttpUserAgent()
{
    $agents = array(
        'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:77.0) Gecko/20190101 Firefox/77.0',
        'Mozilla/5.0 (Windows NT 10.0; WOW64; rv:77.0) Gecko/20100101 Firefox/77.0',
        'Mozilla/5.0 (X11; Linux ppc64le; rv:75.0) Gecko/20100101 Firefox/75.0',
        'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:39.0) Gecko/20100101 Firefox/75.0',
        'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.10; rv:75.0) Gecko/20100101 Firefox/75.0',
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
        'Mozilla/5.0 (X11; Ubuntu; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2919.83 Safari/537.36',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2866.71 Safari/537.36',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.75.14 (KHTML, like Gecko) Version/7.0.3 Safari/7046A194A',
        'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/74.0.3729.169 Safari/537.36',
        'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 Safari/537.36'
    );
    $ua = $agents[rand(0, count($agents) - 1)];
    define('REQUESTING_USER_AGENT', $ua);
    return $ua;
}

/**
 * this function receives something like:
 *   array(3) {
 *     ["type"]=>
 *     string(6) "linked"
 *     ["thumb_url"]=>
 *     string(93) "https://i.pixxxels.cc/pdY4Bvnn/Met-Models-The-World-s-Finest-Nude-Model-Photography-Index.jpg"
 *     ["main_url"]=>
 *     string(28) "https://pixxxels.cc/5HysxZgx"
 *   }
 * @param $image array
 * @return string the name of the host
 */
function getImageHost($image) {
    $rs = parse_url($image['main_url']);
    if ($rs) {
        return $rs['host'];
    }
    return false;
}

function check_session()
{
    if (!empty($_SESSION['current_session'])) {
        return TRUE;
    } else {
        return FALSE;
    }
}

function out($data, $end = true)
{
    echo "Out: ". PHP_EOL;
    var_dump($data);
    echo PHP_EOL;
    if ($end) {
        exit;
    }
}

function getArguments()
{
    $opts = getopt("s::vtu::");
    $retval = array('start' => 0, 'verbose' => null, 'title' => null, 'uri' => null);

    if (array_key_exists("s", $opts)) {
        $retval['start'] = $opts["s"];
    }

    if (array_key_exists("t", $opts)) {
        $retval['title'] = $opts["t"];
    }

    if (array_key_exists("u", $opts)) {
        $retval['uri'] = $opts["u"];
    }

    if (array_key_exists("v", $opts)  && ($opts['v'] == 'true')) {
        $retval['verbose'] = true;
    } elseif(array_key_exists("v", $opts)  && ($opts['v'] == 'false')) {
        $retval['verbose'] = false;
    }
    return $retval;
}

function getArgsTitle($suggested) {
    echo PHP_EOL;
    echo 'Suggested Title: '. $suggested . PHP_EOL;
    echo 'Press [return] to use suggested title or enter a new title.'.PHP_EOL;
    echo 'Title: ';
    $arg = trim(fgets(STDIN));
    if (empty($arg)) {
        $arg = $suggested;
    }
    return $arg;
}


function verbose($string, $verbose = false)
{
    if (!is_string($string)) {
        echo var_export($string, true) .PHP_EOL;
    }
    if ($verbose) {
        echo $string . PHP_EOL;
    }
}

/**
 * @param $set
 * @param $threshold int The number of images that need to be in a set for it to be considered a set.
 */
function isImageSet($set, $threshold) {
    $count = 0;
    $matches = parse_array($set, '<a href', '</a>');
    if ($matches) {
        foreach ($matches as $match) {
            $pos = strpos($match, '<img src="');
            if ($pos !== false) {
                $count++;
            }
        }
    }
    if ($count > $threshold) {
        return $matches;
    } else {
        return false;
    }
}

/**
 * This function attempts to remove image count, dimension and date information from the title.  It makes it easier to
 * use the ripper.  This is a convenience function, if it creates problems it can be safely bypassed.
 * @param $title string Text like: Domai Rena A - Rena A 4 - x33 - 4000x2667  (4 Nov, 2019)
 * @return string should return something like: Domai Rena A - Rena A 4
 */
function cleanUpTitle($title) {
    $title = trim(utf8_encode($title));
    $patterns = array(
        '/\(\d{1,2}\s\w+\,\s\d{4}\)/', // (4 Nov, 2019)
        '/\(\w+\s\d{1,2}\,\s\d{4}\)/', // (Nov 4, 2019)
        '/\s\d+(x|X)/', // <space>120x or <space>120X
        '/\s(x|X)\d+/', // <space>x120 or <space>X120
        '/\s\d+px/', // <space>1280px
        '/-\s\d+\spictures\s-/', // -<space>1280<space>pictures<space>-
        '/\d{2}-\d{2}-\d{4}/', // 04-03-2014
        '/\(\d{2,3}\sphotos\)/', // (75 photos)
        '/\(\d{4}\sX\s\d{4}\)/', // (2592 X 3888)
        '/\d{8}/', //30012008
        '/\(\d{4}-\d{2}-\d{2}\)/', // (2012-02-10)
        '/\d{2,4}\spics/', // 116 pics
        '/\s-\s$/', // -<space>
        '/\s-\s-$/', // -<space>-
        '/\s-$/', // -
// <space>-<space> at end of title
//        '/-\sx\d{2,3}\s-\s\d{4}x\d{4}\s+\(\d{1,2}\s\w+\,\s\d{4}\)/', //Domai Rena A - Rena A 4 - x33 - 4000x2667  (4 Nov, 2019)
//        '/-\s*x\d{2,3}\s-\s\d*px\s+\(\d*\/\d*\/\d*\)/', // -x69 - 3888px (4/4/19)
//        '/-\s\d{2,3}\s\w+\s-\s\d{4}\w+\s\(\d{1,2}\s\w+\s\d{4}\)/', // - 116 Pics - 5600px (03 NOV 2019)
//        '/\s*x\d{2,3}\s\d*px\s+\(\d*\-\d*\-\d*\)/', //  x47 3000px (11-04-2019)
//        '/-\s*\d{2,3}\s\w*\s-\s\d*\smb\s/', // - 87 pics - 104 mb
//        '/-\sx\d*\s-\s\d*px\s+\(\d*\s\w+\,\s\d{4}\)/', //  - x69 - 4000px (4 Nov, 2019)
//        '/\(x\d*\)\s\d*x\d*/' // (x118) 2848x4288
        // Xiuren No.1469 绯月樱-Cherry (x53)
        // MetArt Laina – Tres Cool - 120 pictures - 8688px (23 Nov, 2019)
        // 4913060-Alisa-Amore-–-Top-Mesh-118-pictures-7952px-(2-Dec-2019)
//        '/-x\d{2,3}-\d{4}x\d{4}/', // -x101-1600x1200
//        '/x\d{2,3} \d{4}x\d{4}/', // x101 1600x1200
//        '/-\d{4}x\d{4}/', // -1600x1200
//        '/\d{4}x\d{4}/', // 1600x1200
//        '/-x\d{2,3}/', // -x101
//        '/-\(x\d{2,3}\)/', // -(x101)
//        '/\(x\d{2,3}\)/', // (x101)
//        '/x\d{2,3}/', // x101
    );
    $replace = '';
    foreach ($patterns as $pattern) {
        $retval = preg_replace($pattern, $replace, $title, -1, $count);
        if($count === 1) {
            $title = trim($retval);
        }
    }
    return trim($title);
}
//
//function convertToCharset($string, $charset = 'UTF-8') {
//    return iconv("UTF-8", "ISO-8859-1//TRANSLIT", $string);
//}

function doLikeOfPost($post_id, $hash, $ripper) {
    $url = 'https://vipergirls.to/post_thanks.php?do=post_thanks_add&p='. $post_id .'&securitytoken='. $hash;
    $rs = $ripper->getFileWithCurl($url);
    return $rs;
}

/**
 * should build something like: post_thanks.php?do=post_thanks_add&p=54516061&securitytoken=1579042978-daf3e79ab4cf96a2edd526de6943356a2fbbb0d7
 * where 54516061 is the post ID and the securitytoken changes on every page load, and you have to be logged in otherwise
 * it gets set to 'guest' which doesn't have permissions to like a post.
 *
 * The actual like link in the album is created with javascript/AJAX
 * 
 * @param $html the HTML of the album page
 * @param int $offset
 * @return bool|string
 */
function getLikeLink($html, $offset=0) {
    $rs = array();
    $links = parse_array($html, '<a href', '</a>');
    foreach($links as $link) {
        if (strpos($link, 'post_thanks_add')) {
            $rs[] = $link;
        }
    }
    out($rs);
    if (count($rs) >= 0) {
        $link = $rs[$offset];
        $retval = getUriFromLink($link);
    } else {
        $retval = false;
    }
    return $retval;
}

function getSecurityToken($html) {

}

/**
 * @param $link string HTML href string like: <a href="post_thanks.php?do=post_thanks_add&amp;p=58238840&amp;securitytoken=1579109354-445ac7a82ef7148680bc032d6264f9e5b0ad8d58" id="post_thanks_button_58238840" onclick="return post_thanks_give(58238840, false);" rel="nofollow" class="post_thanks_button">Like</a>
 */
function getUriFromLink($link) {
    $a = new SimpleXMLElement($link);
    return (string)$a['href'];
}

/**
 * expecting an XML document that looks like this:
 * <?xml version="1.0" encoding="utf-8"?>
 *  <images>
 *      <user hash="1588872955-e306b45cbb3b01fe44a182d9bfb5e50bbca522f6" />
 *      <forum title="Adult Photo Collections" />
 *      <thread id="4856487" title="MetModels 2005-2006" />
 *      <post id="54948686" number="1" title="MetModels 2005-2006" imagecount="1" >
 *          <image type="linked" thumb_url="https://i.pixxxels.cc/pdY4Bvnn/Met-Models-The-World-s-Finest-Nude-Model-Photography-Index.jpg" main_url="https://pixxxels.cc/5HysxZgx" />
 *      </post>
 *      <post id="54948925" number="2" title="Re MetModels 2005" imagecount="252" >
 *          <image type="linked" thumb_url="https://i.pixxxels.cc/L5WX2tRH/cover.jpg" main_url="https://pixxxels.cc/" />
 *          <image type="linked" thumb_url="https://t36.pixhost.to/thumbs/135/126228059_metmodels_gg_2_0000.jpg" main_url="https://pixhost.to/show/135/126228059_metmodels_gg_2_0000.jpg" />
 * ...
 * @param $xml
 * @return mixed
 */
function xmlToArray($xml) {
    $counter = 0;
    $retval['hash'] = (string)$xml->user->attributes()->hash;
    $retval['thread_id'] = (string)$xml->thread->attributes()->id;
    $retval['thread_title'] = (string)$xml->thread->attributes()->title;

    $post = $xml->xpath('post');

    foreach ($post as $node) {
        $attrs = $node->attributes();
        foreach ($attrs as $key => $value) {
            $attributes[$key] = (string)$value;
        }
        $retval['posts'][$counter]['attributes'] = $attributes;

        $children = $node->children();
        foreach ($children as $key => $value) {
            $image_attrs = $value->attributes();
            foreach ($image_attrs as $key => $value) {
                $image_attributes[$key] = (string)$value;
            }
            $images[]['attr'] = $image_attributes;
        }
        $retval['posts'][$counter]['images'] = $images;
        $counter++;
//        if($counter == 2){
//            out($retval);
//        }
    }
    return $retval;
}

/**
 * returns an array with information pulled from the XML
 * @param $xml
 * @return mixed
 */
function getThreadInfoFromXml($xml) {
    $counter = 0;
    $retval['hash'] = (string)$xml->user->attributes()->hash;
    $retval['thread_id'] = (string)$xml->thread->attributes()->id;
    $retval['thread_title'] = (string)$xml->thread->attributes()->title;

    $post = $xml->xpath('post');
    foreach ($post as $node) {
        $attrs = $node->attributes();
        foreach ($attrs as $key => $value) {
            $attributes[$key] = (string)$value;
        }

        if ($attributes['imagecount'] == 0) {
            continue;
        }
        $retval['posts'][$counter]['attributes'] = $attributes;

        // get the first image as an example
        $children = $node->children();
        if ($children->count() == 0) {
//            out($attributes);
            continue;
        }

        $attrs = $children[0]->attributes();
        $image['type'] = (string)$attrs->type;
        $image['thumb_url'] = (string)$attrs->thumb_url;
        $image['main_url'] = (string)$attrs->main_url;

        $retval['posts'][$counter]['first_image'] = $image;

        $counter++;
//        if($counter == 2){
//            out($retval);
//        }
    }
    return $retval;
}

function getPostFromXml($xml, $post_id) {
    $retval['hash'] = (string)$xml->user->attributes()->hash;
    $retval['thread_id'] = (string)$xml->thread->attributes()->id;
    $retval['thread_title'] = (string)$xml->thread->attributes()->title;

    $post = $xml->xpath('//post[@id='. $post_id .']');
    foreach($post as $node) {
        $attrs = $node->attributes();
        foreach ($attrs as $key => $value) {
            $attributes[$key] = (string)$value;
        }
        $retval['attributes'] = $attributes;
        $count = 0;
        foreach ($node->image as $image) {
            $attrs = $image->attributes();
            $retval['images'][$count]['type'] = (string)$attrs->type;
            $retval['images'][$count]['thumb_url'] = (string)$attrs->thumb_url;
            $retval['images'][$count]['main_url'] = (string)$attrs->main_url;
            $count++;
        }
    }
    return $retval;
}


function validateUri($uri) {
    $var = filter_var($uri, FILTER_VALIDATE_URL);
    if ($var === false) {
        return false;
    }
    return true;
}

/***********************************************************************
Parse Constants (scope = global)
----------------------------------------------------------------------*/
# Specifies if parse includes the delineator
define("EXCL", true);
define("INCL", false);
# Specifies if parse returns the text before or after the delineator
define("BEFORE", true);
define("AFTER", false);

/***********************************************************************
split_string($string, $delineator, $desired, $type)
-------------------------------------------------------------
DESCRIPTION:
Returns a potion of the string that is either before or after
the delineator. The parse is not case sensitive, but the case of
the parsed string is not effected.
INPUT:
$string         Input string to parse
$delineator     Delineation point (place where split occurs)
$desired        BEFORE: return portion before delineator
AFTER:  return portion before delineator
$type           INCL: include delineator in parsed string
EXCL: exclude delineator in parsed string
 ***********************************************************************/
function split_string($string, $delineator, $desired, $type)
{
    # Case insensitive parse, convert string and delineator to lower case
    $lc_str = strtolower($string);
    $marker = strtolower($delineator);

    # Return text BEFORE the delineator
    if($desired == BEFORE)
    {
        if($type == EXCL)  // Return text EXCL of the delineator
            $split_here = strpos($lc_str, $marker);
        else               // Return text INCL of the delineator
            $split_here = strpos($lc_str, $marker)+strlen($marker);

        $parsed_string = substr($string, 0, $split_here);
    }
    # Return text AFTER the delineator
    else
    {
        if($type==EXCL)    // Return text ESCL of the delineator
            $split_here = strpos($lc_str, $marker) + strlen($marker);
        else               // Return text INCL of the delineator
            $split_here = strpos($lc_str, $marker) ;

        $parsed_string =  substr($string, $split_here, strlen($string));
    }
    return $parsed_string;
}

/***********************************************************************
$value = return_between($string, $start, $end, $type)
-------------------------------------------------------------
DESCRIPTION:
Returns a substring of $string delineated by $start and $end
The parse is not case sensitive, but the case of the parsed
string is not effected.
INPUT:
$string         Input string to parse
$start          Defines the beginning of the sub string
$end            Defines the end of the sub string
$type           INCL: include delineators in parsed string
EXCL: exclude delineators in parsed string
 ***********************************************************************/
function return_between($string, $start, $stop, $type)
{
    $temp = split_string($string, $start, AFTER, $type);
    $temp = split_string($temp, $stop, BEFORE, $type);
    return $temp;
}

/***********************************************************************
$array = parse_array($string, $open_tag, $close_tag)
-------------------------------------------------------------
DESCRIPTION:
Returns an array of strings that exists repeatedly in $string.
This function is usful for returning an array that contains
links, images, tables or any other data that appears more than
once.
INPUT:
$string     String that contains the tags
$open_tag   Name of the open tag (i.e. "<a>")
$close_tag  Name of the closing tag (i.e. "</title>")

 ***********************************************************************/
function parse_array($string, $beg_tag, $close_tag)
{
    preg_match_all("($beg_tag(.*)$close_tag)siU", $string, $matching_data);
    return $matching_data[0];
}

function echo_color($string, $foregroundcolor = null, $backgroundcolor = null) {
    require_once 'TerminalColors.php';
    $t = new TerminalColors();
    echo $t->paint($string, $foregroundcolor, $backgroundcolor);
}

