<?php
/**
 * rip_history.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

if (!defined('PATH_TO_CONFIG')) {
    define('PATH_TO_CONFIG', 'conf/config.php');
}

/**
 * this function allows the user to view their download history
 * @param $args
 * @param $rip
 */
function rip_history($args, $rip)
{
    verbose('starting history', $args['verbose']);

    switch ($args['history']) {
        case 'report':
            $rs = generateReport($args, $rip);
            break;

        case 'clear':
            $rs = clearHistory($args, $rip);
            break;

        default:
            $rs = displayHistory($args, $rip);
            break;
    }
    return $rs;
}

function displayHistory($args, $rip) {
    verbose('starting display history', $args['verbose']);
    $rs = getHistoryData($rip);
    drawTable($rs);
    return true;
}

function getHistoryData($rip) {
    $log_downloads = $rip->getConfigValue('log_downloads');
    if ($log_downloads) {
        $rs = $rip->db->getActivity();
    } else {
        $rs = null;
    }
    return $rs;
}

function drawTable($data) {
    $colAwidth = 50;
    $colBwidth = 90;
    $colCwidth = 9;
    $colDwidth = 21;
    echo '+'. str_pad('', $colAwidth, '-').'+'. str_pad('', $colBwidth, '-').'+'. str_pad('', $colCwidth, '-').'+'. str_pad('', $colDwidth, '-').'+'.PHP_EOL;
    echo '|'. str_pad('  title', $colAwidth). '|' . str_pad('  uri', $colBwidth). '+'. str_pad('  count', $colCwidth).'|'. str_pad('  date', $colDwidth).'|'.PHP_EOL;
    echo '+'. str_pad('', $colAwidth, '-').'+'. str_pad('', $colBwidth, '-').'+'. str_pad('', $colCwidth, '-').'+'. str_pad('', $colDwidth, '-').'+'.PHP_EOL;
    if (is_null($data)) {
        echo '|'. str_pad('  ---', $colAwidth). '|' . str_pad('  ---', $colBwidth). '+'. str_pad('  ---', $colCwidth).'|'. str_pad('  ---', $colDwidth).'|'.PHP_EOL;

    } else {
        foreach ($data as $row) {
            echo '|'. str_pad($row['title'], $colAwidth). '|' . str_pad($row['uri'], $colBwidth). '+'. str_pad('  '.$row['image_count'], $colCwidth).'|'. str_pad('  '.$row['datetime'], $colDwidth).'|'.PHP_EOL;
        }
    }
    echo '+'. str_pad('', $colAwidth, '-').'+'. str_pad('', $colBwidth, '-').'+'. str_pad('', $colCwidth, '-').'+'. str_pad('', $colDwidth, '-').'+'.PHP_EOL;
}

function generateReport($args, $rip) {
    verbose('starting generate report', $args['verbose']);
    $rs = getHistoryData($rip);
    $table = drawHtmlTable($rs);
    $title = 'Laxaeripper History';
    $tpl = reportTemplate();
    $html = str_replace(['%%title%%', '%%contents%%'], [$title, $table], $tpl);
    $rs = writeReport($html);
    if ($rs['rs']) {
        echo 'generated report, you can find it in the history folder in the root of the laxaeripper folder, named '. $rs['file'].PHP_EOL;
        exit;
    }
}

function drawHtmlTable($data) {
    $html = '<table id="history">'.PHP_EOL;
    $html .= '  <tr>'.PHP_EOL;
    $html .= '    <th>title</th><th>uri</th><th>count</th><th>date</th>'.PHP_EOL;
    $html .= '  </tr>'.PHP_EOL;
    foreach($data as $row) {
        $html .= '  <tr>'.PHP_EOL;
        $html .= '    <td>'.$row['title'].'</td><td><a href="'.$row['uri'].'">'.$row['uri'].'</a></td><td>'.$row['image_count'].'</td><td>'.$row['datetime'].'</td>'.PHP_EOL;
        $html .= '  </tr>'.PHP_EOL;
    }
    $html .= '</table>'.PHP_EOL;
    return $html;
}

function reportTemplate() {
    return '<!DOCTYPE html>
<html lang="en">
<head>
    <link href="laxaeripper.css" rel="stylesheet" type="text/css">
    <meta charset="UTF-8">
    <title>%%title%%</title>
</head>
<body>
<h1>%%title%%</h1>
%%contents%%
</body>
</html>';
}

function writeReport($html) {
    $date = date('Y-m-d');
    $destination = 'history-'.$date.'.html';
    $rs = file_put_contents('history/'.$destination, $html);
    $retval['rs'] = $rs;
    $retval['file'] = $destination;
    return $retval;
}

function clearHistory($args, $rip) {
    $rs = $rip->db->clearActivity();
    if ($rs) {
        return 'Cleared activity from database';
    }
    return false;
}