<?php
/**
 * rip.php
 *
 * @version $Id: $
 * @package Ripper
 * @copyright 2019
 */

/**
 * @todo add support for pulling from kittykat: http://kitty-kats.net/threads/kristzy-a-hairdresser.2713137/
 */

$start_time = microtime(true);
if (PHP_OS === 'Darwin') {
    passthru('clear');
}
// have to change the directory to the folder of this script, otherwise will run in some other folder and break
chdir(dirname(__FILE__));
$flag_config = false;
$conf = []; // this will get overwritten by the config file, if it exists
if (file_exists('conf/config.php')) {
    $flag_config = true;
    require_once 'conf/config.php';
}
define('SITE_ROOT', dirname(__FILE__) .'/');

require_once 'app/functions.php';
require_once 'conf/proxies.php';
require_once 'app/BaseHelper.php';
require_once 'app/Ripper.php';
require_once 'app/ViperGirls.php';
require_once 'app/RipperDownloadInterface.php';
require_once 'app/RipperDownload.php';
//require_once 'app/RipperHttpful.php';
require_once 'app/RipperVipergirlsXml.php';

if (array_key_exists('log_downloads', $conf) && ($conf['log_downloads'] == true)) {
    require_once 'app/RipperSQLiteDb.php';
    $db = new RipperSQLiteDb();
} else {
    $db = null;
    $conf['log_downloads'] = false;
}

// the CLI verbose flag -v=true should overwrite the config file value
$args = getArguments();

if (is_null($args['verbose'])) {
    $verbose = $conf['verbose'] ?? false;
} else {
    $verbose = $args['verbose'];
}

if ($verbose) {
    $str = 'true';
    $conf['verbose'] = 'true';
} else {
    $str = 'false';
}
verbose('set verbose to: '. $str, $verbose);

//$rd = new RipperDownload($verbose);
$rd = new RipperVipergirlsXml($verbose);
//$rd = new RipperHttpful($verbose);
$rip = new Ripper();
$rip->config($conf);
$rip->setDownloader($rd);
$rip->setDb($db);
// @todo need to setThreadHost after receiving the URL we want to download images from.
$rip->setThreadHost(new ViperGirls($rip->getConfigValue('username'),$rip->getConfigValue('password'),$rip->getConfigValue('user_agent')));
// @todo need to set username and password for each different Thread_Host that is in use
$rip->getDownloader()->init([
    'username'=>$rip->getConfigValue('username'),
    'password'=>$rip->getConfigValue('password'),
    'user_agent'=>$rip->getConfigValue('user_agent')
]);

$version = $rip->getInstalledVersion();

echo_color('L A X A E R I P P E R '. $version, 'brown', 'white');
if (!$flag_config) {
    echo PHP_EOL;
    echo_color('You have not configured laxaeripper, you should do so before running it.', 'red');
    echo PHP_EOL;
    echo_color('Press the number 8 to enter config', 'red');
    echo PHP_EOL;
}
echo PHP_EOL;
echo '[1] Rip images in thread - one or most posts';
echo PHP_EOL;
echo '[2] Rip images in post';
echo PHP_EOL;
echo '[3] Rip images from multiple threads listed in a file';
echo PHP_EOL;
echo '[4] Rip images from a pasted list';
echo PHP_EOL;
echo '[6] History';
echo PHP_EOL;
echo '[7] Update';
echo PHP_EOL;
echo '[8] Config';
echo PHP_EOL;
echo '[9] Help information';
echo PHP_EOL;
echo 'Select action: ';
$args['action'] = trim(fgets(STDIN));

/**
 * addn is an array that holds additional information specific to the action selected by the user
 */
$addn = [];

switch ($args['action']) {
    case '1':
        //$addn = getArgsForCollection();
        $addn = getArgsForThread();
        break;
    case '2':
        //$addn = getArgsForAlbum();
        $addn = getArgsForPost();
        break;
    case '3':
        $addn = getArgsForAlbumsInFile();
        break;
    case '4':
        $addn = getArgsForList();
        break;
    case '6':
        $addn = getArgsForHistory();
        break;
    case '8':
        break;
    case '9':
        break;
    case '5':
        break;
    case 't':
        break;
    case '7':
        $addn = getArgsForUpdate();
        break;

    default:
        displayError(array('No action selected, or invalid selection'));
        break;
}

$args = array_merge($args, $addn);

$rs = false;

switch ($args['action']) {
    case '1':
        require_once 'app/rip_thread.php';
        $rs = rip_thread($args, $rip);
        break;

    case '2':
        require_once 'app/rip_post.php';
        $rs = rip_post($args, $rip);
        break;

    case '3':
        require_once 'app/rip_albums_in_file.php';
        $rs = rip_albums_in_file($args, $rip);
        break;

    case '4':
        require_once 'app/rip_list.php';
        $rs = rip_list($args, $rip);
        break;

    case '6':
        require_once 'app/rip_history.php';
        $rs = rip_history($args, $rip);
        break;

    case '7':
        require_once 'app/rip_update.php';
        $rs = rip_update($args, $rip);
        break;
    
    case '8':
        require_once 'app/rip_config.php';
        $rs = rip_config($args, $rip);
        break;

    case 't':
        require_once 'app/rip_tests.php';
        $rs = rip_tests($args, $rip);
        break;

    case '5':
    default:
        displayHelp();
        break;

}

echo PHP_EOL;
echo '---DONE---'. PHP_EOL;
$memory = memory_get_peak_usage();
$end_time = microtime(true);
$execution_time = ($end_time - $start_time);
echo 'Script took '. round($execution_time) .' second(s) and used '. round(($memory/1000000),2) .' MB of memory'. PHP_EOL;

if (PHP_OS === 'Darwin') {
    passthru('tput bel');
}

function getArgsForThread() {
    echo PHP_EOL;
    $error = array();
    echo 'uri: ';
    $args['uri'] = trim(fgets(STDIN));
    echo PHP_EOL;

    if (!$args['uri']) {
        $error[] = 'must supply a URI to the list of images: -u "http://example.com/path/to/links"';
    }

    $valid = validateUri($args['uri']);
    if (!$valid) {
        $error[] = 'the URI is invalid';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForPost() {
    echo PHP_EOL;
    $error = array();
    echo 'uri: ';
    $args['uri'] = trim(fgets(STDIN));
    echo PHP_EOL;

    if (!$args['uri']) {
        $error[] = 'must supply a URI to the list of images: -u "http://example.com/path/to/links"';
    }

    $valid = validateUri($args['uri']);
    if (!$valid) {
        $error[] = 'the URI is invalid';
    }

    echo 'start after image #: ';
    $args['start'] = trim(fgets(STDIN));

    if ($args['start'] === '') {
        $args['start'] = 0;
    }
    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForAlbum() {
    echo PHP_EOL;
//    echo 'title: ';
//    $args['title'] = trim(fgets(STDIN));
    echo 'uri: ';
    $args['uri'] = trim(fgets(STDIN));
    echo 'start after image #: ';
    $args['start'] = trim(fgets(STDIN));

    if ($args['start'] === '') {
        $args['start'] = 0;
    }
    $error = array();

    if (!$args['uri']) {
        $error[] = 'must supply a URI to the list of images, like "http://example.com/path/to/album"';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForAlbumsInFile() {
    echo PHP_EOL;
    echo 'filename: ';
    $args['filename'] = trim(fgets(STDIN));

    $error = array();

    if (!$args['filename']) {
        $error[] = 'must supply a filename for the file that contains the list of threads or posts';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForList() {
    echo PHP_EOL;
    echo 'There is a maximum of 1024 characters that can be pasted into the terminal, don\'t paste a large list';
    echo PHP_EOL;
    echo 'title: ';
    $args['title'] = trim(fgets(STDIN));
    echo PHP_EOL;
    echo 'links: ';
    $args['links'] = trim(fgets(STDIN));

    $error = array();

    if (!$args['title']) {
        $error[] = 'must supply a title for the list of images, like "XArt Mia Inside"';
    }
    if (!$args['links']) {
        $error[] = 'must supply a batch of images to process';
    }

    if (count($error) > 0) {
        displayError($error);
    }
    return $args;
}

function getArgsForHistory() {
    echo PHP_EOL;
    echo '[1] view history'.PHP_EOL;
    echo '[2] generate HTML report'. PHP_EOL;
    echo '[x] clear history';
    echo PHP_EOL;
    echo 'Select action: ';
    $action = trim(fgets(STDIN));

    $error = array();

    if (!$action) {
        $error[] = 'Please select an action from the list.';
    }

    if ($action == 2) {
        $args['history'] = 'report';
    } elseif ($action == 'x') {
        $args['history'] = 'clear';
    } else {
        $args['history'] = 'view';
    }
    return $args;
}

function getArgsForUpdate() {
    $args = [];
    return $args;
}

function displayError($error) {
    echo_color('ERROR', 'red');
    echo PHP_EOL;
    foreach($error as $item) {
        echo_color($item, 'red');
        echo PHP_EOL;
    }
    echo PHP_EOL;
    exit;
}


function displayHelp() {
    require_once 'app/RipperHelp.php';
    $help = new RipperHelp();
    $help->displayCliHelp();
}
